<?php
/***********************************************
 * File      :   BackendCalendarExpresso.php
 * Project   :   Z-Push
 * Descr     :   This Calendar Backend is for Expresso Groupware.
 *
 * Created   :   06.12.2010 - emerson-faria.nobre@serpro.gov.br
 *
 * � Zarafa Deutschland GmbH, www.zarafaserver.de
 * This file is distributed under GPL v2.
 * Consult LICENSE file for details
 ************************************************/
include_once('diffbackend.php');
include_once('include/z_RTF.php');
include_once "include/dbconnect.php";

class BackendCalendarExpresso extends BackendDiff {
	var $_user;
	var $_devid;
	var $_protocolversion;
	var $db;

	function __construct()
	{
		$this->db = dbconnect();

		if (!$this->db) {
			echo "A DB connect error occured.\n";
			exit;
		}
	}

	function Setup($user, $devid, $protocolversion) {
		$this->_user = $user;
		$this->_devid = $devid;
		$this->_protocolversion = $protocolversion;
		//TODO: Autenticar Usuario no LDAP

		return true;
	}

	function SendMail($rfc822, $forward = false, $reply = false, $parent = false) {
		return false;
	}

	function GetWasteBasket() {
		return false;
	}

	function GetMessageList($folderid, $cutoffdate) {
		$id_owner = $this->parseOwner($this->_user);
		if ($id_owner == false) return false;

		debugLog('CalendarExpresso::GetMessageList('.$id_owner.')');
		if ($folderid != "calendar") return false;

		$messages = array();
		try {
			$result = pg_query($this->db,"BEGIN;");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			if ($cutoffdate == 0) $result = pg_query($this->db, "select c.cal_id, c.last_update, c.datetime, c.cal_type from phpgw_cal c join phpgw_cal_user u on (c.cal_id = u.cal_id) where u.cal_login ='" . $id_owner . "'and c.reference = 0;");
			else $result = pg_query($this->db, "select c.cal_id, c.last_update, c.datetime, c.cal_type from phpgw_cal c join phpgw_cal_user u on (c.cal_id = u.cal_id) where u.cal_login ='" . $id_owner . "'and c.reference = 0 and c.edatetime > " . $cutoffdate . ";");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			while ($row = pg_fetch_row($result)) {
				$result_recur = pg_query($this->db,"select recur_enddate from phpgw_cal_repeats where cal_id = " . $row[0] . ";");
				if ($result_recur == FALSE) throw new Exception(pg_last_error($this->db));
				if ($row[3] == 'M') {
					if ($recur_enddate = pg_fetch_result($result_recur, 0, 0)) {
						if ($recur_enddate < $row[2]) continue; // Nao sincroniza este evento, porque a data final da recorrencia eh menor que a data inicial do evento. Eh o BD do Expresso que esta inconsistente.
					}
				}
				$LatestUpdate = $row[1];
				$message = array();
				$message["id"] = $row[0];
				$message["mod"] = substr($LatestUpdate, 0, strlen($LatestUpdate)-3);
				$message["flags"] = 1; // always 'read'
				$messages[] = $message;
			}
			$result = pg_query($this->db,"COMMIT;");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
		} catch (Exception $e) {
			pg_query($this->db,"ROLLBACK;");
			debugLog("exception -> " . $e->getMessage() . " - ARQUIVO: " . $e->getFile() . " - LINHA: " . $e->getLine());
		}
		return $messages;
	}

	function GetFolderList() {
		debugLog('CalendarExpresso::GetFolderList()');
		$calendars = array();
		$folder = $this->StatFolder("calendar");
		$calendars[] = $folder;
		//return $calendars;
		return $folder;
	}

	function GetFolder($id) {
		debugLog('CalendarExpresso::GetFolder('.$id.')');
		if($id == "calendar") {
			$folder = new SyncFolder();
			$folder->serverid = $id;
			$folder->parentid = "0";
			$folder->displayname = "Calendario";
			$folder->type = SYNC_FOLDER_TYPE_APPOINTMENT;
			return $folder;
		} else return false;
	}

	function StatFolder($id) {
		debugLog('CalendarExpresso::StatFolder('.$id.')');
		$folder = $this->GetFolder($id);
		$stat = array();
		$stat["id"] = $id;
		$stat["parent"] = $folder->parentid;
		$stat["mod"] = $folder->displayname;
		return $stat;
	}

	function GetAttachmentData($attname) {
		return false;
	}

	function StatMessage($folderid, $id) {
		debugLog('CalendarExpresso::StatMessage('.$folderid.', '.$id.')');
		if($folderid != "calendar") return false;
		try {
		 $result = pg_query($this->db,"BEGIN;");
		 if ($result == FALSE) throw new Exception(pg_last_error($this->db));
		 $result_calendar = pg_query($this->db, "select last_update from phpgw_cal where cal_id = " . $id . ";");
		 if ($result_calendar == FALSE) throw new Exception(pg_last_error($this->db));
		 while ($row_calendar = pg_fetch_row($result_calendar)) {
		 	if(isset($row_calendar[0])) {
		 		$message = array();
		 		$message["mod"] = substr($row_calendar[0], 0, strlen($row_calendar[0])-3);
		 		$message["id"] = $id;
		 		$message["flags"] = 1;
		 		return $message;
		 	}
		 }
		 $result = pg_query($this->db,"COMMIT;");
		 if ($result == FALSE) throw new Exception(pg_last_error($this->db));
		} catch (Exception $e) {
			pg_query($this->db,"ROLLBACK;");
			debugLog("exception -> " . $e->getMessage() . " - ARQUIVO: " . $e->getFile() . " - LINHA: " . $e->getLine());
		}
		return false;
	}

	function GetMessageDAO($id) {
		debugLog('CalendarExpresso::GetMessageDAO('.$id.', ..)');
		if(trim($id == "")) return false;
		$message = new SyncAppointment();
		try {
			$has_parent_event = false;
			$result = pg_query($this->db,"BEGIN;");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			//Evento
			$result = pg_query($this->db,"select config_value from phpgw_config WHERE config_name = 'hostname' AND config_app = 'phpgwapi';");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			$uid = "-@" . pg_fetch_result($result, 0, 0) . "-" . $id;
			$result = pg_query($this->db, "select cal_id, title, location, mdatetime, datetime, edatetime, description, reference, is_public, category from phpgw_cal where cal_id = " . $id . ";");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			if ($row = pg_fetch_row($result)) {
				if ($row[7] != 0) $has_parent_event = true;
				else $has_parent_event = false;
				if (!$has_parent_event) {
					$message->fileas = $id;
					$message->uid = utf8_encode($uid);
					if(isset($row[3])) {
						$tz = date_default_timezone_get();
						date_default_timezone_set('UTC');
						$message->dtstamp = $row[3]; // phpgw_cal.mdatetime
						date_default_timezone_set($tz);
					}
					if(isset($row[4])) {
						$tz = date_default_timezone_get();
						date_default_timezone_set('UTC');
						$message->starttime =  $row[4]; // phpgw_cal.datetime
						date_default_timezone_set($tz);
					}
					if(isset($row[5])) {
						$tz = date_default_timezone_get();
						date_default_timezone_set('UTC');
						$message->endtime = $row[5]; // phpgw_cal.edatetime
						date_default_timezone_set($tz);
					}
				}
				if ($has_parent_event) {
					$message->exceptionstarttime = $row[4];
					$message->deleted = 0;
				}
				if(isset($row[1])) {
					$message->subject = utf8_encode($row[1]); // phpgw_cal.title
				}
				if(isset($row[2])) {
					$message->location = utf8_encode($row[2]); // phpgw_cal.location
				}
				if(isset($row[6]) and $row[6] != "") {
					$message->body = utf8_encode(str_replace('\\n', chr(13) . chr(10), $this->escape($row[6]))); // phpgw_cal.description
					$message->bodysize = strlen($message->body);
					$message->bodytruncated = 0;
				}
				if (isset($row[8]) and $row[8] == 1) $message->sensitivity = 0; // 0 - Normal
				else $message->sensitivity = 2; // 2 - Privado
				if (isset($row[9]) and $row[9] != "") {
					$result = pg_query($this->db,"select cat_name from phpgw_categories WHERE cat_id in(" . $row[9] . ");");
					if ($result == FALSE) throw new Exception(pg_last_error($this->db));
					while ($cat_name = pg_fetch_row($result)) {
						if (!isset($message->categories)) $message->categories = array();
						array_push($message->categories, utf8_encode($cat_name[0]));
					}
				}
				$message->alldayevent = 0; // (0 - No(default), 1- Sim)
				$message->busystatus = 2; // 2 - Ocupado
				//TODO: Tratar $message - timezone
				//$tz = $this->_getGMTTZ();
				$tz = array("bias" => 180, "stdbias" => 0, "dstbias" => -60, "dstendyear" => 0, "dstendmonth" => 2, "dstendday" => 0, "dstendweek" => 2, "dstendhour" => 2, "dstendminute" => 0, "dstendsecond" => 0, "dstendmillis" => 0,
		                                      "dststartyear" => 0, "dststartmonth" =>10, "dststartday" =>0, "dststartweek" => 3, "dststarthour" => 2, "dststartminute" => 0, "dststartsecond" => 0, "dststartmillis" => 0);
				$message->timezone = base64_encode($this->_getSyncBlobFromTZ($tz));
				//TODO: Se necessario, tratar $message - meetingstatus
				//TODO: Se necessario, tratar AppointmentReplyTime e ResponseType
			}

			//Alarme do Evento - O alarme nao sera sincronizado por enquanto.
			/* Esse exemplo sincroniza o alarme mais distante do evento com o dispositivo movel
			* $result = pg_query("select data from phpgw_async where id like 'cal:" . $id . ":%' and data like '%enabled%' order by next desc;");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			if ($data = pg_fetch_result($result, 0, 0)) {
			$array_data = unserialize($data);
			$offset = $array_data["offset"];
			if (isset($array_data["offset"])) {
			if ($offset >= 60) $message->reminder = intval($offset / 60);
			else $message->reminder = 0;
			}
			}*/
			if (!$has_parent_event) {
				//Recorrencia do evento
				//TODO: Implementar recorrencia mensal por dia e anual por dia, recur - type 3 e 6 respectivamente. O Expresso ainda nao suporta isso. :(
				$result = pg_query($this->db,"select recur_type, recur_use_end, recur_enddate, recur_interval, recur_data, recur_exception from phpgw_cal_repeats where cal_id = " . $id . "and recur_type in (1,2,3,5);");
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				if ($row = pg_fetch_row($result)){
					$recur = new SyncRecurrence();
					// Converte os tipos do Expresso para os tipos do Protocolo ActiveSync
					switch ($row[0]) {
						case 1: //repeticao diaria
							$recur->type = 0;
							break;
						case 2: //repeticao semanal
							$recur->type = 1;
							break;
						case 3: //repeticao mensal por data
							$recur->type = 2;
							break;
						case 5: //repeticao anual por data
							$recur->type = 5;
							break;
					}
					if (isset($row[2])) {
						$tz = date_default_timezone_get();
						date_default_timezone_set('UTC');
						$recur->until = $row[2];
						date_default_timezone_set($tz);
					}
					if (isset($row[3]) and $row[3] != 0) $recur->interval = $row[3];
					else $recur->interval = 1;
					//TODO: Se o Expresso estiver utilizando o campo phpgw_cal_repeats.recur_use_end, implementar.
					if ($recur->type == 1 or $recur->type == 3 or $recur->type == 6) {
						if (isset($row[4])) $recur->dayofweek = $row[4];
					}
					if ($recur->type == 2 or $recur->type == 5) {
						$tz = date_default_timezone_get();
						date_default_timezone_set('UTC');
						$array_startdatetime	= getdate($message->starttime);
						$recur->dayofmonth = $array_startdatetime['mday'];
						date_default_timezone_set($tz);

					}
					if ($recur->type == 5 or $recur->type == 6) {
						$tz = date_default_timezone_get();
						date_default_timezone_set('UTC');
						$array_startdatetime	= getdate($message->starttime);
						$recur->monthofyear = $array_startdatetime['mon'];
						date_default_timezone_set($tz);

					}
					$message->recurrence = $recur;
				}

				//Eventos Unicos da Recorrencia Editados
				$result = pg_query($this->db, "select cal_id from phpgw_cal where reference = " . $id . ";");
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				while ($row_recur_exception = pg_fetch_row($result)) {
					$recur_exception = $this->GetMessageDAO($row_recur_exception[0]);
					if ($recur_exception instanceof SyncAppointment) {
						if (!isset($message->exceptions)) $message->exceptions = array();
						array_push($message->exceptions, $recur_exception);
					}
				}

				//Eventos Unicos da Recorrencia Deletados
				if (isset($row[5]) and $row[5] != "") {
					$array_timestamps_recur_deleted = explode(",",$row[5]);
					foreach ($array_timestamps_recur_deleted as $timestamp_recur_deleted) {
						$recur_deleted = new SyncAppointment();
						$recur_deleted->deleted = '1';
						$recur_deleted->exceptionstarttime = (integer)$timestamp_recur_deleted;
						if (!isset($message->exceptions)) $message->exceptions = array();
						array_push($message->exceptions, $recur_deleted);
					}
				}
				//TODO: Tratar Participantes do Evento
			}
			$result = pg_query($this->db,"COMMIT;");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
		} catch (Exception $e) {
			pg_query($this->db,"ROLLBACK;");
			debugLog("exception -> " . $e->getMessage() . " - ARQUIVO: " . $e->getFile() . " - LINHA: " . $e->getLine());
			return;
		}
		return $message;
	}

	function GetMessage($folderid, $id, $truncsize, $mimesupport = 0) {
		debugLog('CalendarExpresso::GetMessage('.$folderid.', '.$id.', ..)');
		if($folderid != "calendar")	return;
		if(trim($id == "")) return;
		$message = $this->GetMessageDAO($id);
		return $message;

		//$message->?????  //phpgw_cal.priority (1 - baixa, 2 - normal, 3 - alta) //Quando for criar um registro o valor ser 2.
		//$message->?????  //phpgw_cal.groups (No vou atribuir nenhum valor)
		//$message->?????  //phpgw_cal.cal_type (quando for criar um novo registro vou inserir R para Recursivo e E para no recursivo). Quando Alterar um registro entre de Recursivo para no Recursivo e vice-versa vou fazer o mesmo da incluso. Quando alterar entre dois tipos no recursivos, no fao nada.
		//$message->?????  //phpgw_cal.ex_participants
		//$message->?????  //phpgw_cal_user.cal_status
		//$message->?????  //phpgw_cal_user.cal_type
		//$message->?????  //phpgw_cal_repeats.recur_interval
		//$message->?????  //phpgw_cal_repeats.recur_data
		//$message->?????  //phpgw_cal_repeats.recur_exception

		//$message->timezone  //phpgw?????
		//$message->organizername //phpgw?????
		//$message->organizeremail //phpgw????
		//$message->attendees // phpgw????  (SyncAttendee array)

		//$syncAttendee = new SyncAttendee();
		//$syncAttendee->email  //phpgw????
		//$syncAttendee->name   //phpgw????
	}

	function DeleteMessage($folderid, $id) {
		debugLog('CalendarExpresso::DeleteMessage('.$folderid.', '.$id.', ..)');
		if (!isset($id)) return false;
		$id_owner = $this->parseOwner($this->_user);
		if ($id_owner == false) return false;
		$result = pg_query($this->db, "select owner from phpgw_cal where cal_id = " . $id . ";");
		if ($result == FALSE) throw new Exception(pg_last_error($this->db));
		if ($id_owner != pg_fetch_result($result, 0, 0)) return false;
		try {
			$result = pg_query($this->db,"BEGIN;");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			$result = pg_delete($this->db, "phpgw_cal_user", array('cal_id' => $id));
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			$result = pg_delete($this->db, "phpgw_cal", array('reference' => $id));
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			$result = pg_delete($this->db, "phpgw_cal_repeats", array('cal_id' => $id));
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			$result = pg_query("delete from phpgw_async where id like 'cal:" . $id . "%';");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			$result = pg_delete($this->db, "phpgw_cal_extra", array('cal_id' => $id));
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			$result = pg_delete($this->db, "phpgw_cal", array('cal_id' => $id));
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			$result = pg_query($this->db,"COMMIT;");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
		} catch (Exception $e) {
			pg_query($this->db,"ROLLBACK;");
			debugLog("exception -> " . $e->getMessage() . " - ARQUIVO: " . $e->getFile() . " - LINHA: " . $e->getLine());
			return false;
		}
		return true;
	}

	function SetReadFlag($folderid, $id, $flags) {
		return false;
	}

	function ChangeMessageDAO($id, $message) {
		debugLog('CalendarExpresso::ChangeMessageDAO('.$id.', ..)');
		$id_owner = $this->parseOwner($this->_user);
		if ($id_owner == false) return false;
		try {
			$result = pg_query($this->db,"BEGIN;");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			$found_cal_id = false;
			$found_cal_user = false;
			$found_cal_repeats = false;
			if ($id != false) {
				$result = pg_query($this->db, "select cal_id, cal_type, owner from phpgw_cal where cal_id = " . $id . ";");
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				// tenta localizar cal_id para fazer Update
				while ($row = pg_fetch_row($result)) {
					if(isset($row[0])) {
						if ($id_owner != $row[2]) return false; // Nao atualiza evento de outro proprietario
						$cal_id = $row[0];
						$cal_type_from_DB = $row[1];
						$found_cal_id = true;
					}
				}
				// Verifica se ja existe o registro do evento na tabela phpgw_cal_user
				$result = pg_query($this->db, "select cal_id, cal_login from phpgw_cal_user where cal_id = " . $cal_id . " and cal_login = '" . $id_owner . "';");
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				if ($row = pg_fetch_row($result)) {
					if(isset($row[0])) {
						$found_cal_user = true;
					}
				}
				// Verifica se ja existe o registro de recorrencia do evento na tabela phpgw_cal_repeats
				$result = pg_query($this->db, "select cal_id from phpgw_cal_repeats where cal_id = " . $cal_id . ";");
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				if ($row = pg_fetch_row($result)) {
					if(isset($row[0])) {
						$found_cal_repeats = true;
					}
				}
			}

			//Insere/Atualiza tabela phpgw_cal
			if(isset($message->uid)) {
				$arrayCal["uid"] = $this->truncateString(utf8_decode($message->uid),255);
			}
			if(isset($message->subject)) {
				$arrayCal["title"] = $this->truncateString(utf8_decode($message->subject),80);
			}
			if(isset($message->location)) {
				$arrayCal["location"] = $this->truncateString(utf8_decode($message->location),255);
			}
			if(isset($message->dtstamp)) {
				$arrayCal["mdatetime"] = $message->dtstamp;
			}
			if(isset($message->starttime)) {
				$arrayCal["datetime"] = $message->starttime;
			}
			if(isset($message->endtime)) {
				$arrayCal["edatetime"] = $message->endtime;
			}
			if(isset($message->sensitivity) and $message->sensitivity == 0) {
				$arrayCal["is_public"] = 1; // 1 - Normal
			} else $arrayCal["is_public"] = 0; // 0 - Privado

			if(isset($message->rtf)) {
				$rtf_to_ascii = new rtf();
				$rtf_to_ascii->output("ascii");
				$result_loadrtf = $rtf_to_ascii->loadrtf(base64_decode($message->rtf));
				if ($result_loadrtf == true) $rtf_to_ascii->parse();
				$arrayCal["description"] = $rtf_to_ascii->out;
//			} else {
//				$arrayCal["description"] = '';
			}
			$arrayCal["category"] = '';
			if (isset($message->categories)) {
				foreach ($message->categories as $category) {
					$cat_id = $this->addCategory($category);
					if ($cat_id != false) {
						if ($arrayCal["category"] == '') {
							$arrayCal["category"] = $cat_id;
						} else {
							$arrayCal["category"] .= ',' . $cat_id;
						}
					}
				}
			}
			$tz_CEL = $this->_getTZFromSyncBlob(base64_decode($message->timezone));
			//TODO: Atribuir valores para ex_participants
			$arrayCal["ex_participants"] = '';
			//TODO: Converter eventos que sao alldayevent para iniciar as 0:00hs e terminar as 23:59hs. Isso porque o Expresso nao suporta alldayevents
			//		 if(isset($message->alldayevent)) {
			//		 	$arrayCal["alldayevent"] = $this->truncateString(utf8_decode($message->alldayevent),30);
			//		 }
			if (!$found_cal_id){
				$result = pg_query("select nextval('seq_phpgw_cal');");
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				$cal_id = pg_fetch_result($result, 0, 0);
				$arrayCal["cal_id"] = $cal_id;
				$arrayCal["owner"] = $id_owner;
				if (isset($message->recurrence)) $arrayCal["cal_type"] = 'M';
				else $arrayCal["cal_type"] = 'E';
				$result = pg_insert($this->db, 'phpgw_cal', $arrayCal);
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			} else {
				if (isset($cal_type_from_DB)) {
					if (isset($message->recurrence)) $arrayCal["cal_type"] = 'M';
					else if ($cal_type_from_DB == 'M') $arrayCal["cal_type"] = 'E';
				}
				$result = pg_update($this->db, 'phpgw_cal', $arrayCal, array('cal_id' => $cal_id));
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			}

			// Insere/Atualiza tabela phpgw_cal_user
			$arrayCalUser["cal_type"] = 'u';
			if (!$found_cal_user){
				$arrayCalUser["cal_id"] = $cal_id;
				$arrayCalUser["cal_login"] = $id_owner;
				//TODO: Tratar os outros valores de cal_status
				$arrayCalUser["cal_status"] = 'A';
				$result = pg_insert($this->db, 'phpgw_cal_user', $arrayCalUser);
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			} else {
				$result = pg_update($this->db, 'phpgw_cal_user', $arrayCalUser, array('cal_id' => $cal_id, 'cal_login' => $id_owner));
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			}
			if (!$id) {
				$id = $cal_id;
			}

			//TODO: Implementar para os tipos 3 e 6. O Expresso ainda nao suporta esses tipos :-(
			if (isset($message->recurrence) and ($message->recurrence->type == 0 or $message->recurrence->type == 1 or $message->recurrence->type == 2 or $message->recurrence->type == 5)) {
				switch ($message->recurrence->type) {
					case 0: //repeticao diaria
						$arrayRecur["recur_type"] = 1;
						$type_name = "day";
						break;
					case 1: //repeticao semanal
						$arrayRecur["recur_type"] = 2;
						$type_name = "week";
						break;
					case 2: //repeticao mensal por data
						$arrayRecur["recur_type"] = 3;
						$type_name = "month";
						break;
					case 5: //repeticao anual por data
						$arrayRecur["recur_type"] = 5;
						$type_name = "year";
						break;
				}
				if (isset($message->recurrence->interval)) {
					$arrayRecur["recur_interval"] = $message->recurrence->interval;
				}
				if ($message->recurrence->type == 1 or $message->recurrence->type == 3 or $message->recurrence->type == 6) {
					if (isset($message->recurrence->dayofweek)) {
						$arrayRecur["recur_data"] = $message->recurrence->dayofweek;
						/*$day = $message->recurrence->dayofweek;
						 $day_of_week_counter = 0;
						 if (($day & 1) > 0) $day_of_week_counter++;
						 if (($day & 2) > 0) $day_of_week_counter++;
						 if (($day & 4) > 0) $day_of_week_counter++;
						 if (($day & 8) > 0) $day_of_week_counter++;
						 if (($day & 16) > 0) $day_of_week_counter++;
						 if (($day & 32) > 0) $day_of_week_counter++;
						 if (($day & 64) > 0) $day_of_week_counter++;*/
					}
				}
				//TODO: Converter ocurrences para until.
				/*if (!isset($message->recurrence->until) and (isset($message->recurrence->occurrences))) {
				$tz = date_default_timezone_get();
				date_default_timezone_set('UTC');
				$message->recurrence->until = strtotime(date("Y-m-d H:i:s", $message->starttime) . " +" . $message->recurrence->occurrences * $message->recurrence->interval . " " . $type_name);
				unset($message->recurrence->occurrences);
				date_default_timezone_set($tz);
				}*/
				if (isset($message->recurrence->until)){
					$defaultTZ = new DateTimeZone(date_default_timezone_get());
					$gmtTZ = new DateTimeZone('GMT');
					$gmtDate = new DateTime("now", $gmtTZ);
					$GMT_EX = $defaultTZ->getOffset($gmtDate);
					$GMT_CEL = -(($tz_CEL["bias"] + $tz_CEL["dstbias"]) * 60);

					$tz = date_default_timezone_get();
					date_default_timezone_set('UTC');
					$day_CEL = date("d", $message->starttime +  $GMT_CEL);
					$day_EX = date("d", $message->starttime +  $GMT_EX);

					if ($day_EX > $day_CEL) $arrayRecur["recur_enddate"] = $message->recurrence->until + $GMT_CEL - $GMT_EX + 86400;
					else if ($day_EX < $day_CEL) $arrayRecur["recur_enddate"] = $message->recurrence->until + $GMT_CEL - $GMT_EX - 86400;
					else $arrayRecur["recur_enddate"] = $message->recurrence->until;
					date_default_timezone_set($tz);
					/*					//$arrayRecur["recur_enddate"] = $message->recurrence->until - $offsetTZ;
					 $hour = date("G", $message->starttime - (($tz["bias"] - $tz["dstbias"]) * 60));
					 $min = date("i", $message->starttime - (($tz["bias"] - $tz["dstbias"]) * 60));
					 $sec = date("s", $message->starttime - (($tz["bias"] - $tz["dstbias"]) * 60));
					 	
					 $arrayRecur["recur_enddate"] = $message->recurrence->until + ($hour * 3600) + ($min * 60) + $sec + $offsetTZ + (($tz["bias"] - $tz["dstbias"]) * 60);*/
				} else {
					$arrayRecur["recur_enddate"] = 1893283200; // Se nao tem data de termino seta para 30/12/2029
				}

				// Trata excecoes da recorrencia
				if (isset($message->exceptions)){
					//if ($recur_exception_changed_starttimes != "") {
					//Tem Excecoes de recorrencia do tipo Edicao
					// Verifica se ja existe o registro do evento de excecao de recorrencia na tabela phpgw_cal
					//foreach ($array_recur_exception_changed_starttime as $except_starttime) {
					$recur_exception_changed_ids = "";
					$recur_exception_deleted_starttimes = "";
					foreach ($message->exceptions as $recur_exception) {
						if ($recur_exception->deleted == 0){ //Tem Excecoes do Tipo Edicao
							$found_recur_exception_changed = true;
							if (isset($arrayExceptCal)) unset($arrayExceptCal);
							if (isset($arrayExceptCalUser)) unset($arrayExceptCalUser);
							if (isset($except_cal_id)) unset($except_cal_id);
							if (isset($except_cal_type_from_DB)) unset($except_cal_type_from_DB);
							$found_except_cal_id = false;
							$found_except_cal_user = false;
							$result = pg_query($this->db, "select cal_id, cal_type from phpgw_cal where reference = " . $id . " and datetime = " . $recur_exception->exceptionstarttime . ";");
							if ($result == FALSE) throw new Exception(pg_last_error($this->db));
							// tenta localizar cal_id para fazer Update
							while ($row = pg_fetch_row($result)) {
								if(isset($row[0])) {
									$except_cal_id = $row[0];
									$except_cal_type_from_DB = $row[1];
									$found_except_cal_id = true;
								}
							}
							// Verifica se ja existe o registro do evento de excecao de recorrencia na tabela phpgw_cal_user
							if (isset($except_cal_id)) {
								$result = pg_query($this->db, "select cal_id, cal_login from phpgw_cal_user where cal_id = " . $except_cal_id . " and cal_login = '" . $id_owner . "';");
								if ($result == FALSE) throw new Exception(pg_last_error($this->db));
								if ($row = pg_fetch_row($result)) {
									if(isset($row[0])) {
										$found_except_cal_user = true;
									}
								}
							}
							//Insere/Atualiza tabela phpgw_cal
							$arrayExceptCal["reference"] = $cal_id;
							if(isset($message->uid)) {
								$arrayExceptCal["uid"] = $this->truncateString(utf8_decode($message->uid),255);
							}
							if(isset($recur_exception->subject)) {
								$arrayExceptCal["title"] = $this->truncateString(utf8_decode($recur_exception->subject),80);
							} elseif(isset($message->subject)) {
								$arrayExceptCal["title"] = $this->truncateString(utf8_decode($message->subject),80);
							}
							if(isset($recur_exception->location)) {
								$arrayExceptCal["location"] = $this->truncateString(utf8_decode($recur_exception->location),255);
							} else if(isset($message->location)) {
								$arrayExceptCal["location"] = $this->truncateString(utf8_decode($message->location),255);
							}
							if(isset($recur_exception->dtstamp)) {
								$arrayExceptCal["mdatetime"] = $recur_exception->dtstamp;
							} elseif(isset($message->dtstamp)) {
								$arrayExceptCal["mdatetime"] = $message->dtstamp;
							}
							if(isset($recur_exception->starttime)) {
								$arrayExceptCal["datetime"] = $recur_exception->starttime;
							} else if(isset($message->starttime)) {
								$arrayExceptCal["datetime"] = $message->starttime;
							}
							if(isset($recur_exception->endtime)) {
								$arrayExceptCal["edatetime"] = $recur_exception->endtime;
							}elseif(isset($message->endtime)) {
								$arrayExceptCal["edatetime"] = $message->endtime;
							}
							if(isset($recur_exception->sensitivity) and $recur_exception->sensitivity == 0) {
								$arrayExceptCal["is_public"] = 1; // 1 - Normal
							}elseif (isset($recur_exception->sensitivity) and $recur_exception->sensitivity == 1){
								$arrayExceptCal["is_public"] = 0; // 0 - Privado
							}elseif(isset($message->sensitivity) and $message->sensitivity == 0) {
								$arrayExceptCal["is_public"] = 1; // 1 - Normal
							}else $arrayExceptCal["is_public"] = 0; // 0 - Privado
							if(isset($recur_exception->rtf) or isset($message->rtf)) {
								$rtf_to_ascii = new rtf();
								$rtf_to_ascii->output("ascii");
								if(isset($recur_exception->rtf)) $result_loadrtf = $rtf_to_ascii->loadrtf(base64_decode($recur_exception->rtf));
								elseif(isset($message->rtf)) $result_loadrtf = $rtf_to_ascii->loadrtf(base64_decode($message->rtf));
								if ($result_loadrtf == true) $rtf_to_ascii->parse();
								$arrayExceptCal["description"] = $rtf_to_ascii->out;
				//			} else {
				//				$arrayExceptCal["description"] = '';
							}
							$arrayExceptCal["category"] = '';
							if (isset($recur_exception->categories)) {
								foreach ($recur_exception->categories as $category) {
									$cat_id = $this->addCategory($category);
									if ($cat_id != false) {
										if ($arrayExceptCal["category"] == '') {
											$arrayExceptCal["category"] = $cat_id;
										} else {
											$arrayExceptCal["category"] .= ',' . $cat_id;
										}
									}
								}
							}elseif (isset($message->categories)) {
								foreach ($message->categories as $category) {
									$cat_id = $this->addCategory($category);
									if ($cat_id != false) {
										if ($arrayExceptCal["category"] == '') {
											$arrayExceptCal["category"] = $cat_id;
										} else {
											$arrayExceptCal["category"] .= ',' . $cat_id;
										}
									}
								}
							}
							//TODO: Converter eventos que sao alldayevent para iniciar as 0:00hs e terminar as 23:59hs. Isso porque o Expresso nao suporta alldayevents
							//		 if(isset($message->alldayevent)) {
							//		 	$arrayCal["alldayevent"] = $this->truncateString(utf8_decode($message->alldayevent),30);
							//		 }
							if (!$found_except_cal_id){
								$result = pg_query("select nextval('seq_phpgw_cal');");
								if ($result == FALSE) throw new Exception(pg_last_error($this->db));
								$except_cal_id = pg_fetch_result($result, 0, 0);
								$arrayExceptCal["cal_id"] = $except_cal_id;
								$arrayExceptCal["owner"] = $id_owner;
								$arrayExceptCal["cal_type"] = 'E';
								$result = pg_insert($this->db, 'phpgw_cal', $arrayExceptCal);
								if ($result == FALSE) throw new Exception(pg_last_error($this->db));
							} else {
								$result = pg_update($this->db, 'phpgw_cal', $arrayExceptCal, array('cal_id' => $except_cal_id));
								if ($result == FALSE) throw new Exception(pg_last_error($this->db));
							}

							// Insere/Atualiza Excecoes de Recorrencia na tabela phpgw_cal_user
							$arrayExceptCalUser["cal_type"] = 'u';
							if (!$found_except_cal_user){
								$arrayExceptCalUser["cal_id"] = $except_cal_id;
								$arrayExceptCalUser["cal_login"] = $id_owner;
								//TODO: Tratar os outros valores de cal_status
								$arrayExceptCalUser["cal_status"] = 'A';
								$result = pg_insert($this->db, 'phpgw_cal_user', $arrayExceptCalUser);
								if ($result == FALSE) throw new Exception(pg_last_error($this->db));
							} else {
								$result = pg_update($this->db, 'phpgw_cal_user', $arrayExceptCalUser, array('cal_id' => $except_cal_id, 'cal_login' => $id_owner));
								if ($result == FALSE) throw new Exception(pg_last_error($this->db));
							}
							if ($recur_exception_changed_ids == "") {
								$recur_exception_changed_ids = $except_cal_id;
							} else {
								$recur_exception_changed_ids .= "," . $except_cal_id;
							}
						} else {
							if ($recur_exception_deleted_starttimes == "") {
								$recur_exception_deleted_starttimes = $recur_exception->exceptionstarttime;
							} else {
								$recur_exception_deleted_starttimes .= "," . $recur_exception->exceptionstarttime;
							}
						}
					}
					if ($recur_exception_deleted_starttimes != "") {
						//Tem Excecoes de recorrencia do tipo Exclusao
						$arrayRecur["recur_exception"] = $recur_exception_deleted_starttimes;
					} else {
						$arrayRecur["recur_exception"] = "";
					}
					if ($recur_exception_changed_ids == "") {
						// Se nao tem excecoes de recorrencia do tipo edicao, entao deleta as excecoes da recorrencia
						$result = pg_query($this->db, "delete from phpgw_cal_user where cal_id in (select cal_id from phpgw_cal where reference = " . $cal_id . ");");
						if ($result == FALSE) throw new Exception(pg_last_error($this->db));
						$result = pg_query($this->db, "delete from phpgw_cal where reference = " . $cal_id . ";");
						if ($result == FALSE) throw new Exception(pg_last_error($this->db));
					} else {
						//Deleta as excecoes da recorrencia, exceto as inseridas/atualizadas acima
						$result = pg_query($this->db, "delete from phpgw_cal_user where cal_id in (select cal_id from phpgw_cal where reference = " . $cal_id . " and cal_id not in (" . $recur_exception_changed_ids . "));");
						if ($result == FALSE) throw new Exception(pg_last_error($this->db));
						$result = pg_query($this->db, "delete from phpgw_cal where reference = " . $cal_id . " and cal_id not in (" . $recur_exception_changed_ids . ");");
						if ($result == FALSE) throw new Exception(pg_last_error($this->db));
					}
				} else {
					// Se nao tem excecoes da recorrencia entao
					$arrayRecur["recur_exception"] = "";
					$result = pg_query($this->db, "delete from phpgw_cal_user where cal_id in (select cal_id from phpgw_cal where reference = " . $cal_id . ");");
					if ($result == FALSE) throw new Exception(pg_last_error($this->db));
					$result = pg_query($this->db, "delete from phpgw_cal where reference = " . $cal_id . ";");
					if ($result == FALSE) throw new Exception(pg_last_error($this->db));

				}
				if (!$found_cal_repeats){
					$arrayRecur["cal_id"] = $cal_id;
					$result = pg_insert($this->db, 'phpgw_cal_repeats', $arrayRecur);
					if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				} else {
					$result = pg_update($this->db, 'phpgw_cal_repeats', $arrayRecur, array('cal_id' => $cal_id));
					if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				}
			} elseif ($found_cal_id) {
				// Se nao tem recorrencia entao
				$result = pg_query($this->db, "delete from phpgw_cal_user where cal_id in (select cal_id from phpgw_cal where reference = " . $cal_id . ");");
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				$result = pg_query($this->db, "delete from phpgw_cal where reference = " . $cal_id . ";");
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				$result = pg_query($this->db, "delete from phpgw_cal_repeats where cal_id = " . $cal_id . ";");
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			}
			$result = pg_query($this->db,"COMMIT;");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			//TODO: Implementar Participantes
		} catch (Exception $e) {
			debugLog("exception -> " . $e->getMessage() . " - ARQUIVO: " . $e->getFile() . " - LINHA: " . $e->getLine());
			pg_query($this->db,"ROLLBACK;");
			return false;
		}
		return $id;
	}

	function ChangeMessage($folderid, $id, $message) {
		debugLog('CalendarExpresso::ChangeMessage('.$folderid.', '.$id.', ..)');
		$id = $this->ChangeMessageDAO($id, $message);
		if ($id != false) return $this->StatMessage($folderid, $id);
		else return false;
	}

	function MoveMessage($folderid, $id, $newfolderid) {
		return false;
	}

	function addCategory($category) {
		$cat_id = false;
		$id_owner = $this->parseOwner($this->_user);
		if ($id_owner == false) return false;
		try {
			$result = pg_query($this->db,"BEGIN;");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			$result = pg_query("SELECT cat_id from phpgw_categories WHERE cat_owner = -1 AND cat_appname = 'calendar' AND LOWER(to_ASCII(cat_name)) = '" . trim(strtolower($this->removeAccents(utf8_decode($category)))) . "';");
			if ($result == FALSE) throw new Exception(pg_last_error($this->db));
			if (!($cat_id = pg_fetch_result($result, 0, 0))){
				$result = pg_query("SELECT cat_id from phpgw_categories WHERE cat_owner = " . $id_owner . " AND cat_appname = 'calendar' AND LOWER(to_ASCII(cat_name)) = '" . trim(strtolower($this->removeAccents(utf8_decode($category)))) . "';");
				if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				if (!($cat_id = pg_fetch_result($result, 0, 0))){
					$result = pg_query("select nextval('seq_phpgw_categories');");
					if ($result == FALSE) throw new Exception(pg_last_error($this->db));
					$cat_id = pg_fetch_result($result, 0, 0);
					$arrayCat["cat_id"] = $cat_id;
					$arrayCat["cat_owner"] = $id_owner;
					$arrayCat["cat_access"] = 'public';
					$arrayCat["cat_appname"] = 'calendar';
					$arrayCat["cat_name"] = utf8_decode($category);
					$arrayCat["cat_description"] = '';
					$arrayCat["cat_data"] = 'N;';
					$arrayCat["last_mod"] = time();
					$result = pg_insert($this->db, 'phpgw_categories', $arrayCat);
					if ($result == FALSE) throw new Exception(pg_last_error($this->db));
				}
			}
		} catch (Exception $e) {
			pg_query($this->db,"ROLLBACK;");
			debugLog("exception -> " . $e->getMessage() . " - ARQUIVO: " . $e->getFile() . " - LINHA: " . $e->getLine());
			return false;
		}
		return $cat_id;
	}

	// -----------------------------------

	function escape($data){
		if (is_array($data)) {
			foreach ($data as $key => $val) {
				$data[$key] = $this->escape($val);
			}
			return $data;
		}
		$data = str_replace("\r\n", "\n", $data);
		$data = str_replace("\r", "\n", $data);
		$data = str_replace(array('\\', ';', ',', "\n"), array('\\\\', '\\;', '\\,', '\\n'), $data);
		//return u2wi($data);
		return $data;
	}

	function unescape($data){
		$data = str_replace(array('\\\\', '\\;', '\\,', '\\n','\\N'),array('\\', ';', ',', "\n", "\n"),$data);
		return $data;
	}

	function  removeAccents($data){
		$data = strtr($data,"","aaaaaaaceeeeiiii noooooxouuutbaaaaaaaceeeeiiii nooooo/ouuuty");
		return $data;
	}

	function truncateString($string, $size)
	{
		if(strlen($string) <= $size) return $string;
		else return substr($string, 0, $size - 1);
	}

	function parseOwner($id_owner)
	{
		$pos = strripos($id_owner, '\\');
		if ($pos === false) {
			if (strlen($id_owner) <= 2) return false;
			else return substr($id_owner, 0, strlen($id_owner)-2);
		} else {
			if (strlen($id_owner) <= $pos + 3) {
				return false;
			} else {
				return substr(substr($id_owner, $pos + 1), 0, strlen(substr($id_owner, $pos + 1))-2);
			}
		}
	}

	function _getGMTTZ() {
		//$tz = array("bias" => 0, "stdbias" => 0, "dstbias" => 0, "dstendyear" => 0, "dstendmonth" => 2, "dstendday" => 0, "dstendweek" => 2, "dstendhour" => 2, "dstendminute" => 0, "dstendsecond" => 0, "dstendmillis" => 0,
		//                              "dststartyear" => 0, "dststartmonth" =>10, "dststartday" =>0, "dststartweek" => 3, "dststarthour" => 2, "dststartminute" => 0, "dststartsecond" => 0, "dststartmillis" => 0);
		$tz = array("bias" => 120, "stdbias" => 0, "dstbias" => -60, "dstendyear" => 0, "dstendmonth" => 2, "dstendday" => 0, "dstendweek" => 2, "dstendhour" => 2, "dstendminute" => 0, "dstendsecond" => 0, "dstendmillis" => 0,
		                                      "dststartyear" => 0, "dststartmonth" =>10, "dststartday" =>0, "dststartweek" => 3, "dststarthour" => 2, "dststartminute" => 0, "dststartsecond" => 0, "dststartmillis" => 0);
		return $tz;
	}

	// Unpack timezone info from Sync
	function _getTZFromSyncBlob($data) {
		$tz = unpack(    "lbias/a64name/vdstendyear/vdstendmonth/vdstendday/vdstendweek/vdstendhour/vdstendminute/vdstendsecond/vdstendmillis/" .
                        "lstdbias/a64name/vdststartyear/vdststartmonth/vdststartday/vdststartweek/vdststarthour/vdststartminute/vdststartsecond/vdststartmillis/" .
                        "ldstbias", $data);

		// Make the structure compatible with class.recurrence.php
		$tz["timezone"] = $tz["bias"];
		$tz["timezonedst"] = $tz["dstbias"];

		return $tz;
	}

	// Pack timezone info for Sync
	function _getSyncBlobFromTZ($tz) {
		$packed = pack("la64vvvvvvvv" . "la64vvvvvvvv" . "l",
		$tz["bias"], "", 0, $tz["dstendmonth"], $tz["dstendday"], $tz["dstendweek"], $tz["dstendhour"], $tz["dstendminute"], $tz["dstendsecond"], $tz["dstendmillis"],
		$tz["stdbias"], "", 0, $tz["dststartmonth"], $tz["dststartday"], $tz["dststartweek"], $tz["dststarthour"], $tz["dststartminute"], $tz["dststartsecond"], $tz["dststartmillis"],
		$tz["dstbias"]);

		return $packed;
	}
};
?>